<?php

namespace Drupal\access_policy;

use Drupal\access_policy\Plugin\AccessRule\AccessRulePluginInterface;

/**
 * Access rule definition class.
 *
 * This class consolidates information about an access rule plugin. For example,
 * the entity type and bundles are part of the access policy while the field
 * name is part of the access rule configuration. This consolidates that
 * information together into an immutable object so that it is easier to work
 * with.
 */
class AccessRuleDefinition {

  /**
   * The unique access rule plugin id.
   *
   * @var string
   */
  protected $id;

  /**
   * The access rule plugin.
   *
   * @var \Drupal\access_policy\Plugin\AccessRule\AccessRulePluginInterface
   */
  protected $accessRulePlugin;

  /**
   * The plugin definition.
   *
   * @var array
   */
  protected $pluginDefinition;

  /**
   * The plugin configuration.
   *
   * @var array
   */
  protected $pluginConfig;

  /**
   * The target entity type.
   *
   * @var string
   */
  protected $targetEntityTypeId;

  /**
   * The target bundles.
   *
   * @var array
   */
  protected $targetBundles = [];

  /**
   * Constructs a new access rule definition object.
   *
   * @param \Drupal\access_policy\Plugin\AccessRule\AccessRulePluginInterface $access_rule
   *   The access rule plugin.
   * @param string $entity_type
   *   The entity type.
   * @param array $bundles
   *   Array of bundles.
   */
  public function __construct(AccessRulePluginInterface $access_rule, $entity_type, array $bundles) {
    $this->accessRulePlugin = $access_rule;
    $this->id = $access_rule->getId();
    $this->targetEntityTypeId = $entity_type;
    $this->targetBundles = $bundles;
    $this->pluginDefinition = $access_rule->getPluginDefinition();
    $this->pluginConfig = $access_rule->getConfiguration();
  }

  /**
   * Create a new definition from an existing access rule.
   *
   * @param \Drupal\access_policy\Plugin\AccessRule\AccessRulePluginInterface $access_rule
   *   The access rule plugin.
   *
   * @return static
   *   Access rule definition object.
   */
  public static function createFromAccessRule(AccessRulePluginInterface $access_rule) {
    $policy = $access_rule->getAccessPolicy();
    $acces_rule_definition = new static($access_rule, $policy->getTargetEntityTypeId(), $policy->getTargetBundles());
    return $acces_rule_definition;
  }

  /**
   * Get the unique access rule id.
   *
   * @return string
   *   The unique access rule id.
   */
  public function getId() {
    return $this->id;
  }

  /**
   * Get the plugin definition.
   *
   * @return array
   *   Plugin definition array.
   */
  public function getDefinition() {
    return $this->pluginDefinition;
  }

  /**
   * Get a plugin setting.
   *
   * @param string $setting_name
   *   The setting name.
   *
   * @return mixed
   *   The setting value.
   */
  public function getSetting($setting_name) {
    return $this->pluginConfig[$setting_name];
  }

  /**
   * Get the plugin settings.
   *
   * @return array
   *   Array of access rule configuration.
   */
  public function getSettings() {
    return $this->pluginConfig;
  }

  /**
   * Get the field name.
   *
   * @return string
   *   The field name.
   */
  public function getFieldName() {
    return $this->accessRulePlugin->getFieldName();
  }

  /**
   * Get the field type.
   *
   * @return string
   *   The field type.
   */
  public function getFieldType() {
    return $this->accessRulePlugin->getFieldType();
  }

  /**
   * Get the target entity type id.
   *
   * @return string
   *   The entity type id.
   */
  public function getTargetEntityTypeId() {
    return $this->targetEntityTypeId;
  }

  /**
   * Get the target bundles.
   *
   * @return array
   *   Indexed array of bundles.
   */
  public function getTargetBundles() {
    return $this->targetBundles;
  }

}
