<?php

namespace Drupal\access_policy;

use Drupal\access_policy\Entity\AccessPolicyInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Access rule plugin manager.
 */
class AccessRulePluginManager extends DefaultPluginManager {

  /**
   * Array of instantiated plugins.
   *
   * @var array
   */
  protected $instances = [];

  /**
   * AccessRulePluginManager constructor.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/AccessRule', $namespaces, $module_handler, 'Drupal\access_policy\Plugin\AccessRule\AccessRulePluginInterface', 'Drupal\access_policy\Annotation\AccessRule');

    $this->alterInfo('access_rule');
    $this->setCacheBackend($cache_backend, 'access_rule');
  }

  /**
   * Get the applicable access rule plugins.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $policy
   *   The current access policy.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   *
   * @return array
   *   Array of access rule plugins.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getApplicableAccessRulePlugins(AccessPolicyInterface $policy, EntityInterface $entity) {
    $applicable_plugins = [];
    $rules = $policy->getAccessRules();
    foreach ($rules as $id => $plugin_config) {
      $plugin = $this->getOrCreateInstance($policy, $id);

      if ($plugin->isApplicable($entity)) {
        $applicable_plugins[$id] = $plugin;
      }
    }

    return $applicable_plugins;
  }

  /**
   * Get all access rule plugins on a policy.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $policy
   *   The current access policy.
   *
   * @return array
   *   Array of access rule plugins.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getAccessRulePlugins(AccessPolicyInterface $policy) {
    $plugins = [];
    $rules = $policy->getAccessRules();
    foreach ($rules as $id => $plugin_config) {
      $plugins[$id] = $this->getOrCreateInstance($policy, $id);
    }

    return $plugins;
  }

  /**
   * Get or create instance of access rule plugin.
   *
   * This helps to reduce the number of initializations of an access rule
   * plugin.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $policy
   *   The current access policy.
   * @param string $id
   *   The unique plugin id.
   */
  private function getOrCreateInstance(AccessPolicyInterface $policy, $id) {
    $instance_id = $this->getInstanceId($policy, $id);
    if (isset($this->instances[$instance_id])) {
      $plugin = $this->instances[$instance_id];
    }
    else {
      $plugin_config = $policy->getAccessRule($id);
      $settings = $plugin_config['settings'] ?? [];
      $plugin = $this->createInstance($plugin_config['plugin_id'], $settings);
      $plugin->initialize($id, $policy);
      $this->instances[$instance_id] = $plugin;
    }

    return $plugin;
  }

  /**
   * Get the instance id.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $policy
   *   The access policy.
   * @param string $id
   *   The access rule id.
   *
   * @return string
   *   The instance id.
   */
  private function getInstanceId(AccessPolicyInterface $policy, $id) {
    return $policy->id() . ':' . $id;
  }

  /**
   * Alter the access rule definitions.
   *
   * This replaces the class with the applicable access rule class. This is used
   * to support polymorphic field value access rules.
   *
   * @param array $definitions
   *   Array of access rule plugin definitions.
   */
  public function alterDefinitions(&$definitions) {
    parent::alterDefinitions($definitions);
    $field_map = [];

    foreach ($definitions as $id => $definition) {
      if (isset($definition['base_plugin_id'])) {
        $base_plugin_id = $definition['base_plugin_id'];
        if (isset($definition['field_types'])) {
          foreach ($definition['field_types'] as $field_type) {
            $field_map[$base_plugin_id][$field_type] = $definition['class'];
          }
        }
        unset($definitions[$id]);
      }
    }
    // Override the class based on the field map.
    foreach ($definitions as $id => $definition) {
      if (isset($definition['field_type'])) {
        // Get the base plugin id.
        $explode = explode(':', $id);
        $base_plugin_id = $explode[0];
        $field_type = $definition['field_type'];
        if (!empty($base_plugin_id) && isset($field_map[$base_plugin_id][$field_type])) {
          $definitions[$id]['class'] = $field_map[$base_plugin_id][$field_type];
        }
      }
    }
  }

}
