<?php

namespace Drupal\access_policy\Form;

use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\access_policy\Entity\AccessPolicyInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;

/**
 * The form for adding access rules to an access policy.
 *
 * @internal
 */
class AccessPolicyAddRuleForm extends AccessRuleConfigureFormBase {

  /**
   * The access policy entity object.
   *
   * @var \Drupal\access_policy\Entity\AccessPolicyInterface
   */
  protected $accessPolicy;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'access_policy_add_rule_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, AccessPolicyInterface $access_policy = NULL) {
    $this->accessPolicy = $access_policy;
    $form['#attributes']['class'][] = 'access-rule-modal-form';

    $form['filter'] = [
      '#title' => $this->t('Search'),
      '#type' => 'textfield',
      '#ajax' => [
        'event' => 'change',
        'keypress' => TRUE,
        'callback' => [$this, 'ajaxFilterCallback'],
      ],
    ];

    $form['access_rules'] = [
      '#type' => 'fieldset',
      '#tree' => TRUE,
    ];

    // If we're configuring a rule, load the form instead.
    if ($this->isConfiguring($form, $form_state)) {
      $form = $this->buildConfigureForm($form, $form_state);
      return $form;
    }

    // Show access rule plugins (if available).
    $definitions = $this->getApplicableAccessRules();

    // Filter the access rules if applicable.
    if (!empty($form_state->getValue('filter'))) {
      $definitions = $this->filterAccessRules($definitions, $form_state->getValue('filter'));
    }

    if (empty($definitions)) {
      $form['access_rules']['empty'] = [
        '#markup' => $this->t('No access rules found.'),
      ];
    }

    foreach ($definitions as $id => $access_rule) {
      $form['access_rules'][$id] = [
        '#type' => 'checkbox',
        '#title' => $access_rule['label'],
        '#description' => $access_rule['description'],
      ];
    }

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#value' => $this->t('Add rule'),
      // Only validate access rules.
      '#limit_validation_errors' => [['access_rules']],
      '#submit' => ['::addRuleSubmit'],
      '#ajax' => [
        'callback' => [$this, 'ajaxSubmitCallback'],
        'disable-refocus' => TRUE,
      ],
    ];
    $form['actions']['submit']['#validate'][] = [$form_state->getFormObject(), 'validateForm'];

    $form['actions']['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#ajax' => [
        'callback' => [$this, 'ajaxCancelCallback'],
      ],
      '#validate' => [],
      '#limit_validation_errors' => [],
    ];

    return parent::buildForm($form, $form_state, $access_policy);
  }

  /**
   * Build the configure form.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The form.
   */
  public function buildConfigureForm(array $form, FormStateInterface $form_state) {
    $input = $form_state->getUserInput();
    $new_form_state = new FormState();
    $args = func_get_args();
    // Remove $form_arg from the arguments.
    unset($args[0]);
    $new_form_state->addBuildInfo('args', ['access_policy' => $this->accessPolicy, 'id' => $form_state->get('id'), 'is_new' => TRUE]);
    $new_form_state->setValues($input);
    return \Drupal::formBuilder()->buildForm(AccessPolicyConfigureRuleForm::class, $new_form_state);
  }

  /**
   * Update rule submission handler.
   *
   * This is essentially a wrapper for the submission handler inside
   * AccessPolicyConfigureRuleForm::updateRuleSubmit.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public function updateRuleSubmit(array &$form, FormStateInterface $form_state) {
    $input = $form_state->getUserInput();
    $form_state->setValues($input);
    $form_state->cleanValues();
    $form_state->addBuildInfo('args', ['access_policy' => $this->accessPolicy, 'id' => $form_state->get('id'), 'is_new' => TRUE]);
    \Drupal::formBuilder()->submitForm(AccessPolicyConfigureRuleForm::class, $form_state);
  }

  /**
   * Determine whether we're in the middle of configuring a new rule plugin.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return bool
   *   TRUE if the access rule is being configured; FALSE otherwise.
   */
  public function isConfiguring(array $form, FormStateInterface $form_state) {
    $input = $form_state->getUserInput();
    // If the id is present that means we're configuring the access
    // rule.
    if (isset($input['id']) || ($form_state->get('id') && $form_state->get('is_new'))) {
      // Make sure that the id is up-to-date. It can be unset during validation.
      if (!empty($input['id'])) {
        $form_state->set('id', $input['id']);
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Add rule submission handler.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public function addRuleSubmit(array &$form, FormStateInterface $form_state) {
    $plugins = array_filter($form_state->getValue('access_rules'));
    foreach ($plugins as $plugin_id => $enabled) {
      if ($enabled) {
        $this->startConfiguring($plugin_id, $form_state);
        // Only support one rule right now.
        break;
      }
    }

    // Rebuild the form and load the configuration form.
    $form_state->setRebuild();
  }

  /**
   * Flag the form that we're currently configuring the access rule.
   *
   * @param string $plugin_id
   *   The plugin id.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function startConfiguring($plugin_id, FormStateInterface $form_state) {
    $form_state->set('id', $plugin_id);
    $form_state->set('is_new', TRUE);
  }

  /**
   * Ajax callback to reload the form with access rule configuration options.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   An ajax response object.
   */
  public function ajaxSubmitCallback(array $form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('.access-rule-modal-form', $form));
    return $response;
  }

  /**
   * Ajax callback to reload the form filtered access rules.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   An ajax response object.
   */
  public function ajaxFilterCallback(array $form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('.access-rule-modal-form', $form));
    return $response;
  }

  /**
   * Filter access rules.
   *
   * @param array $definitions
   *   Array of access rule definitions.
   * @param string $search
   *   The search string.
   *
   * @return array
   *   Array of filtered definitions.
   */
  public function filterAccessRules(array $definitions, $search) {
    return array_filter($definitions, function ($definition) use ($search) {
      if (stripos($definition['label'], $search) !== FALSE) {
        return TRUE;
      }
      return FALSE;
    });
  }

  /**
   * Returns all applicable access rules for this policy.
   *
   * @return array
   *   Array of applicable access rules.
   */
  public function getApplicableAccessRules() {
    $definitions = $this->accessRulePluginManager->getDefinitions();
    $entity_type_id = $this->accessPolicy->getTargetEntityTypeId();
    return array_filter($definitions, function ($definition) use ($entity_type_id) {
      if (!empty($definition['entity_types'])) {
        return in_array($entity_type_id, $definition['entity_types']);
      }
      return TRUE;
    });

  }

}
