<?php

namespace Drupal\access_policy\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\access_policy\Entity\AccessPolicyInterface;

/**
 * The form for configuring access rules on an access policy.
 *
 * @internal
 */
class AccessPolicyConfigureRuleForm extends AccessRuleConfigureFormBase {

  /**
   * The Access Rule id.
   *
   * @var string
   */
  protected $id;

  /**
   * Flag to determine whether we're configuring a new access rule.
   *
   * @var bool
   */
  protected $isNew;

  /**
   * The access policy entity object.
   *
   * @var \Drupal\access_policy\Entity\AccessPolicyInterface
   */
  protected $accessPolicy;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'access_policy_configure_rule_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, AccessPolicyInterface $access_policy = NULL, $id = NULL, $is_new = NULL) {
    $this->accessPolicy = $access_policy;
    $this->id = $id;
    $this->isNew = $is_new;

    // We need to ensure that the plugin id is preserved during validation.
    // This is passed up to the add rule form when we're configuring an access
    // rule for the first time.
    $form['id'] = [
      '#type' => 'hidden',
      '#value' => $id,
    ];

    // Show any validation errors in the modal.
    $form['status_messages'] = [
      '#type' => 'status_messages',
      '#weight' => -10,
    ];

    // This is necessary for modal validation.
    $form['#attributes']['class'][] = 'access-rule-modal-form';

    // If this is a new rule then a unique id doesn't exist yet. We'll default
    // to using the plugin id.
    if ($this->isNew) {
      $plugin_settings['plugin_id'] = $id;
    }
    // Otherwise use the unique id.
    else {
      $plugin_settings = $access_policy->getAccessRule($this->id);
    }

    $settings = $plugin_settings['settings'] ?? [];

    $plugin_object = $this->accessRulePluginManager->createInstance($plugin_settings['plugin_id'], $settings);
    $plugin_object->initialize($this->id, $access_policy);
    $form += $plugin_object->buildConfigurationForm($form, $form_state);

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['update_rule'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#value' => $this->t('Save'),
      '#submit' => ['::updateRuleSubmit'],
      '#ajax' => [
        'callback' => [$this, 'ajaxSubmitCallback'],
        'disable-refocus' => TRUE,
      ],
    ];

    $form['actions']['submit']['#validate'][] = [$form_state->getFormObject(), 'validateForm'];

    $form['actions']['cancel'] = [
      '#type' => 'button',
      '#value' => $this->t('Cancel'),
      '#ajax' => [
        'callback' => [$this, 'ajaxCancelCallback'],
      ],
      '#validate' => [],
      '#limit_validation_errors' => [],
    ];

    return parent::buildForm($form, $form_state, $access_policy);
  }

  /**
   * Update rule submission handler.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   */
  public function updateRuleSubmit(array &$form, FormStateInterface $form_state) {
    // If this is new, then add the access rule and use that id.
    if ($this->isNew) {
      $id = $this->accessPolicy->addAccessRule($this->id);
    }
    // Otherwise update the existing access rule.
    else {
      $id = $this->id;
    }

    $current_config = $this->accessPolicy->getAccessRule($id);
    $settings = $current_config['settings'] ?? [];

    $plugin = $this->accessRulePluginManager->createInstance($current_config['plugin_id'], $settings);
    $plugin->submitConfigurationForm($form, $form_state);
    $new_settings = $plugin->getConfiguration();
    $settings = array_merge($settings, $new_settings);

    $access_rules = $this->accessPolicy->getAccessRules();
    $access_rules[$id]['settings'] = $settings;
    $this->accessPolicy->setAccessRules($access_rules);
    $this->accessPolicy->save();
  }

}
