<?php

namespace Drupal\access_policy\Form;

use Drupal\access_policy\Entity\AccessPolicyInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\access_policy\AccessRulePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\access_policy\AccessPolicyInformation;

/**
 * Class AccessPolicyEditForm.
 *
 * The AccessPolicy edit form.
 *
 * @package Drupal\access_policy\Form
 */
class AccessPolicyEditForm extends EntityForm {

  use AccessRuleTableTrait;

  /**
   * The AccessRulePluginManager service.
   *
   * @var \Drupal\access_policy\AccessRulePluginManager
   */
  protected $accessRulePluginManager;

  /**
   * The access policy information service.
   *
   * @var \Drupal\access_policy\AccessPolicyInformation
   */
  protected $accessPolicyInformation;

  /**
   * Constructs a AccessPolicyEdiForm object.
   *
   * @param \Drupal\access_policy\AccessRulePluginManager $access_rule_plugin_manager
   *   The AccessRulePluginManager service.
   * @param \Drupal\access_policy\AccessPolicyInformation $access_policy_information
   *   The access policy information service.
   */
  public function __construct(AccessRulePluginManager $access_rule_plugin_manager, AccessPolicyInformation $access_policy_information) {
    $this->accessRulePluginManager = $access_rule_plugin_manager;
    $this->accessPolicyInformation = $access_policy_information;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.access_rule'),
      $container->get('access_policy.information')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $entity = $this->entity;

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $entity->label(),
      '#description' => $this->t("The human-readable name of this access policy. This name must be unique."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\access_policy\Entity\AccessPolicy::load',
      ],
      '#disabled' => !$entity->isNew(),
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Description'),
      '#default_value' => $entity->getDescription(),
      '#description' => $this->t('This text will be displayed on the Access page when this policy is chosen.'),
    ];

    if ($entity->getTypePlugin()->supportsRules()) {
      $form['access_rule_container'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Access Rules'),
      ];

      $form['access_rule_container']['access_rules_table'] = $this->buildAccessRuleTable($entity);

      $form['access_rule_container']['add_rule'] = [
        '#type' => 'link',
        '#title' => $this->t('Add rule'),
        '#url' => Url::fromRoute('access_policy.access_policy_add_rule_form', ['access_policy' => $entity->id()]),
        '#attributes' => $this->getModalAttributes(),
      ];

      $form['access_rule_container']['access_rule_operator'] = [
        '#type' => 'select',
        '#title' => $this->t('Validation operator'),
        '#description' => $this->t('How many rules should pass before a user can access the content?'),
        '#default_value' => $entity->getAccessRuleOperator(),
        '#options' => [
          'AND' => 'All',
          'OR' => 'Any',
        ],
      ];
    }

    $form['query_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Query settings'),
      '#open' => TRUE,
    ];
    $form['query_settings']['query'] = [
      '#type' => 'checkbox',
      '#default_value' => $entity->isQueryEnabled(),
      '#title' => $this->t('Apply policy to listing pages.'),
      '#description' => $this->t('This will hide content on listing pages such as on the admin content page.'),
    ];

    $header = [
      'type' => $this->t('Bundles'),
      'operations' => $this->t('Operations'),
    ];

    $form['bundles_container'] = [
      '#type' => 'details',
      '#title' => $this->t('Bundle selection'),
      '#open' => TRUE,
      '#tree' => TRUE,
    ];

    $form['bundles_container']['entity_type_auto_select'] = [
      '#title' => $this->t('Automatic'),
      '#type' => 'checkbox',
      '#default_value' => $entity->isAutomaticEntityTypeSelection(),
      '#description' => $this->t('Automatically inherit all the bundles of the other policies. Useful for global policies.'),
    ];

    $form['bundles_container']['states_container'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [':input[name="bundles_container[entity_type_auto_select]"]' => ['checked' => FALSE]],
      ],
    ];

    $form['bundles_container']['states_container']['target_bundles'] = [
      '#type' => 'table',
      '#header' => $header,
      '#empty' => $this->t('There are no bundles.'),
    ];
    $selected_bundles_list = [
      '#theme' => 'item_list',
      '#items' => $this->getSelectedBundles($entity),
      '#context' => ['list_style' => 'comma-list'],
      '#empty' => $this->t('None'),
    ];

    $form['bundles_container']['states_container']['target_bundles']['select'] = [
      'type' => [
        '#type' => 'inline_template',
        '#template' => '<span id="selected-bundles">{{ selected_bundles }}',
        '#context' => [
          'entity_type_id' => $entity->getTargetEntityTypeId(),
          'selected_bundles' => $selected_bundles_list,
        ],
      ],
      'operations' => [
        '#type' => 'operations',
        '#links' => [
          'select' => [
            'title' => $this->t('Select'),
            'url' => Url::fromRoute('access_policy.access_policy_bundles_edit_form', ['access_policy' => $entity->id()]),
            'attributes' => $this->getModalAttributes(),
          ],
        ],
      ],
    ];

    return $form;
  }

  /**
   * Load the selected bundles.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $access_policy
   *   The access policy entity.
   *
   * @return array
   *   Array of selected entities.
   */
  public function getSelectedBundles(AccessPolicyInterface $access_policy) {
    $entity_type = $access_policy->getTargetEntityTypeId();
    $bundleInfo = \Drupal::service('entity_type.bundle.info');

    $selected_bundles = [];
    $bundle_info = $bundleInfo->getBundleInfo($entity_type);
    foreach ($bundle_info as $id => $bundle) {
      if ($this->isSelectedBundle($access_policy, $id)) {
        $selected_bundles[$id] = $bundle['label'];
      }
    }

    return $selected_bundles;
  }

  /**
   * Determine whether a bundle is enabled for this entity type.
   *
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $access_policy
   *   The access policy entity.
   * @param string $bundle
   *   The name of the bundle.
   *
   * @return bool
   *   TRUE if it is enabled; FALSE otherwise.
   */
  public function isSelectedBundle(AccessPolicyInterface $access_policy, $bundle) {
    $bundles = $access_policy->getTargetBundles();
    if (in_array($bundle, $bundles)) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $entity = $this->entity;

    $values = $form_state->getValues();
    $entity->setAutomaticEntityTypeSelection($values['bundles_container']['entity_type_auto_select']);

    $status = $entity->save();

    switch ($status) {
      case SAVED_NEW:
        $this->messenger()->addMessage($this->t('Created the %label Access Policy.', [
          '%label' => $entity->label(),
        ]));
        break;

      default:
        $this->messenger()->addMessage($this->t('Saved the %label Access Policy.', [
          '%label' => $entity->label(),
        ]));
    }
    $form_state->setRedirectUrl($entity->toUrl('collection'));
  }

}
