<?php

namespace Drupal\access_policy\Plugin\AccessPolicyType;

/**
 * Access policy types.
 *
 * These plugins control how policies are validated. They control whether
 * permissions and/or rules should be validated for each operation (e.g view,
 * update and delete).
 *
 * This is useful in cases where you want to create a policy but a "view"
 * permission restriction might not make sense for it (like with Public
 * content).
 */
interface AccessPolicyTypeInterface {

  /**
   * Determine whether permissions are validated as part of this type.
   *
   * @param string $op
   *   The operation (e.g view, update, and delete).
   *
   * @return bool
   *   TRUE if permissions should be validated; FALSE otherwise.
   */
  public function shouldValidatePermission($op);

  /**
   * Determine whether rules are validated as part of this type.
   *
   * @param string $op
   *   The operation (e.g view, update and delete).
   *
   * @return bool
   *   TRUE if rules should be validated; FALSE otherwise.
   */
  public function shouldValidateRules($op);

  /**
   * Return the supported operations.
   *
   * @return string[]
   *   Array of operations (e.g view, update, delete, assign).
   */
  public function getAllOperations();

  /**
   * Determine whether this type supports permission validation.
   *
   * @return bool
   *   TRUE if it's supported; FALSE otherwise.
   */
  public function supportsPermissions();

  /**
   * Determine whether this type supports rule validation.
   *
   * @return bool
   *   TRUE if it's supported; FALSE otherwise.
   */
  public function supportsRules();

  /**
   * Determine whether this operation should validate type access rules.
   *
   * @return bool
   *   TRUE if the type rules should be validated; FALSE otherwise.
   */
  public function shouldValidateTypeRules($op);

  /**
   * Get the type access rule ids.
   *
   * @return array
   *   Array of access rule ids.
   */
  public function getTypeRuleIds($op);

  /**
   * Get the type access rule configuration.
   *
   * @return array
   *   Array of access rule configuration.
   */
  public function getTypeRules();

  /**
   * Determine whether this type has access rules defined.
   *
   * @return bool
   *   TRUE if type rules exist; FALSE otherwise.
   */
  public function hasTypeRules();

}
