<?php

namespace Drupal\access_policy\Plugin\AccessRule;

use Drupal\access_policy\AccessRuleDefinition;
use Drupal\access_policy\Entity\AccessPolicyInterface;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;

/**
 * Access rule plugin trait.
 *
 * @package Drupal\access_policy\Plugin\AccessRule
 */
trait AccessRulePluginTrait {

  /**
   * The access rule unique id.
   *
   * @var string
   *   The unique id of this access rule.
   */
  protected $id;

  /**
   * The top level access policy.
   *
   * @var \Drupal\access_policy\Entity\AccessPolicyInterface
   */
  protected $accessPolicy;

  /**
   * Configuration array.
   *
   * @var array
   */
  protected $configuration;

  /**
   * Initialize the access rule plugin.
   *
   * @param string $id
   *   The unique access rule id.
   * @param \Drupal\access_policy\Entity\AccessPolicyInterface $access_policy
   *   The access policy.
   */
  public function initialize($id, AccessPolicyInterface $access_policy) {
    $this->id = $id;
    $this->accessPolicy = $access_policy;
  }

  /**
   * Get the unique id of the access rule.
   *
   * Note that this shouldn't be confused with plugin id. This is the unique
   * id created when added to an access policy.
   *
   * @return string
   *   The access rule unique id.
   */
  public function getId() {
    return $this->id;
  }

  /**
   * Retrieve the Access Policy context.
   *
   * @return \Drupal\access_policy\Entity\AccessPolicyInterface
   *   The Access Policy context.
   */
  public function getAccessPolicy() {
    return $this->accessPolicy;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfiguration() {
    return $this->configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    $this->configuration = NestedArray::mergeDeep(
      $this->baseConfigurationDefaults(),
      $this->defaultConfiguration(),
      $configuration
    );
  }

  /**
   * Returns generic default configuration for access rule plugins.
   *
   * @return array
   *   An associative array with the default configuration.
   */
  protected function baseConfigurationDefaults() {
    return [
      'admin_label' => '',
      'query' => TRUE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function setConfigurationValue($key, $value) {
    $this->configuration[$key] = $value;
  }

  /**
   * {@inheritdoc}
   *
   * Creates a generic configuration form for all access rules. Individual
   * access rule plugins can add elements to this form by overriding
   * AccessRule::accessRuleForm(). Most access rule plugins should not override
   * this method unless they need to alter the generic form elements.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    // Add plugin-specific settings for this access rule type.
    $form += $this->accessRuleForm($form, $form_state);

    // Show the widget settings if supported by the plugin.
    $definition = $this->getPluginDefinition();
    if (!empty($definition['access_rule_widget'])) {
      $form['widget'] = [
        '#type' => 'details',
        '#title' => $this->t('Widget settings'),
        '#open' => FALSE,
        '#tree' => TRUE,
      ];
      $form['widget']['show'] = [
        '#type' => 'checkbox',
        '#default_value' => $this->configuration['widget']['show'] ?? FALSE,
        '#title' => $this->t('Show widget'),
        '#description' => $this->t('Show the widget on the Access tab.'),
      ];
      $form['widget']['settings'] = [
        '#type' => 'container',
        '#states' => [
          'visible' => [':input[name="widget[show]"]' => ['checked' => TRUE]],
        ],
      ];
      $form['widget']['settings'] = $this->buildWidgetSettingsForm($definition['access_rule_widget'], $form['widget']['settings'], $form_state);
    }

    $form['query_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Query settings'),
      '#open' => FALSE,
    ];
    $form['query_settings']['query'] = [
      '#type' => 'checkbox',
      '#default_value' => $this->configuration['query'],
      '#title' => $this->t('Apply rule to listing pages (experimental).'),
      '#description' => $this->t('This will hide content on listing pages such as on the admin content page.'),
    ];

    $form['admin_label'] = [
      '#type' => 'details',
      '#title' => $this->t('Administrative title'),
      '#open' => FALSE,
    ];

    $form['admin_label']['admin_label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Administrative title'),
      '#default_value' => $this->configuration['admin_label'],
      '#description' => $this->t('This will be displayed on the rules list. This is useful when you have the same access rule used twice.'),
    ];

    return $form;
  }

  /**
   * Build the access rule widget settings form.
   *
   * @param string $plugin_id
   *   The widget plugin id.
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current form state.
   *
   * @return array
   *   The access rule widget settings form.
   */
  public function buildWidgetSettingsForm($plugin_id, array $form, FormStateInterface $form_state) {
    $accessRuleWidgetPluginManager = \Drupal::service('plugin.manager.access_rule_widget');
    $widget = $accessRuleWidgetPluginManager->createInstance($plugin_id);
    $definition = AccessRuleDefinition::createFromAccessRule($this);
    $widget->initialize($definition);
    $settings = $this->configuration['widget']['settings'] ?? [];
    $widget->setWidgetSettings($settings);
    return $widget->settingsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * Most access rule plugins should not override this method. To add validation
   * for a specific access rule, override AccessRule::accessRuleFormValidate().
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->accessRuleFormValidate($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * Most access rule plugins should not override this method. To add submission
   * handling for a specific access rule, override
   * AccessRule::accessRuleFormSubmit().
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    // Process the access rule's submission handling if no errors occurred only.
    if (!$form_state->getErrors()) {
      $form_state->cleanValues();
      if (!empty($form_state->getValue('widget'))) {
        $this->configuration['widget'] = $form_state->getValue('widget');
      }
      $this->configuration['admin_label'] = $form_state->getValue('admin_label');
      $this->configuration['query'] = $form_state->getValue('query');
      $this->accessRuleFormSubmit($form, $form_state);
    }
  }

  /**
   * Load any handlers on this plugin.
   *
   * @param string $name
   *   The handler name.
   *
   * @return \Drupal\access_policy\AccessRuleHandlerInterface|false
   *   The access rule handler.
   */
  public function getHandler($name) {
    $definition = $this->getPluginDefinition();
    if (!empty($definition['handlers'][$name])) {
      $class = $definition['handlers'][$name];
      $handler = \Drupal::service('class_resolver')->getInstanceFromDefinition($class);
      $definition = AccessRuleDefinition::createFromAccessRule($this);
      $handler->initialize($definition);
      return $handler;
    }
    return FALSE;
  }

}
