<?php

namespace Drupal\access_policy\Plugin\AccessRule;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\field\FieldStorageConfigInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Field value base class.
 */
class FieldValueBase extends AccessRuleBase implements AccessRuleFieldInterface {

  use AccessRuleOperatorTrait;
  use AccessRuleFieldTrait;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $fieldTypeManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a FieldMatchAccessRule object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_manager
   *   The field type plugin manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityFieldManagerInterface $entity_field_manager, FieldTypePluginManagerInterface $field_type_manager, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityFieldManager = $entity_field_manager;
    $this->fieldTypeManager = $field_type_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(EntityInterface $entity) {
    if ($entity->hasField($this->pluginDefinition['field_name'])) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'target' => 'current_user',
      'user_field' => '',
      'value' => '',
      'empty_behavior' => 'allow',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function accessRuleForm(array $form, FormStateInterface $form_state) {

    $form['operator'] = [
      '#type' => 'select',
      '#title' => $this->t('Operator'),
      '#options' => $this->getOperatorOptions(),
      '#default_value' => $this->configuration['operator'],
      '#description' => $this->t('Select how you want to compare the field values.'),
    ];

    $form['target'] = [
      '#type' => 'select',
      '#title' => $this->t('Compare field values with:'),
      '#default_value' => $this->configuration['target'],
      '#options' => [
        'current_user' => $this->t('Current user'),
        'value' => $this->t('Value'),
      ],
      '#description' => $this->t('Choose what you want to compare, either a field on the the current user or a particular value.'),
    ];

    $form['user_field'] = [
      '#type' => 'select',
      '#title' => $this->t('User field'),
      '#description' => $this->t('Select the field on the user that you want to compare.'),
      '#options' => $this->getUserFieldOptions(),
      '#default_value' => $this->configuration['user_field'],
      '#empty_option' => $this->t('- Select -'),
      '#states' => [
        'visible' => [
          'select[name="target"]' => ['value' => 'current_user'],
        ],
      ],
    ];

    $form['field_value'] = [
      '#type' => 'container',
      '#states' => [
        'visible' => [
          'select[name="target"]' => ['value' => 'value'],
        ],
      ],
    ];
    $form['field_value']['value'] = $this->getFieldWidget($form, $form_state);

    $form['empty'] = [
      '#type' => 'details',
      '#title' => $this->t('Empty behavior'),
    ];
    $form['empty']['empty_behavior'] = [
      '#type' => 'select',
      '#title' => $this->t('Empty behavior'),
      '#options' => [
        'allow' => $this->t('Allow'),
        'deny' => $this->t('Deny'),
      ],
      '#default_value' => $this->configuration['empty_behavior'],
      '#description' => $this->t('Allow or deny access if the field does not have a value.'),
    ];

    return $form;
  }

  /**
   * Create array of operator options.
   *
   * @return array
   *   Array of operators.
   */
  public function getOperatorOptions() {
    $options = [];
    $operators = $this->operators();
    foreach ($operators as $op => $item) {
      $options[$op] = $item['title'];
    }

    return $options;
  }

  /**
   * Array of operators.
   *
   * @return array
   *   The array of operators.
   */
  public function operators() {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldName() {
    return $this->pluginDefinition['field_name'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldType() {
    return $this->pluginDefinition['field_type'];
  }

  /**
   * {@inheritdoc}
   */
  public function accessRuleFormSubmit(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $this->configuration['target'] = $values['target'];
    $this->configuration['user_field'] = $values['user_field'];
    $this->configuration['value'] = $this->extractFieldWidgetValues($form, $form_state);
    $this->configuration['operator'] = $values['operator'];
    $this->configuration['empty_behavior'] = $values['empty_behavior'];
  }

  /**
   * Get options array for user field.
   *
   * @return array
   *   The options array.
   */
  private function getUserFieldOptions() {
    $user_field_definitions = $this->entityFieldManager->getFieldDefinitions('user', 'user');
    return $this->filterByApplicableFieldStorage('user', $user_field_definitions, [$this->pluginDefinition['field_type']]);
  }

  /**
   * Build the field value widget.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The fully formed field widget.
   */
  protected function getFieldWidget(array $form, FormStateInterface $form_state) {
    $field_name = $this->pluginDefinition['field_name'];
    $entity = $this->createStubEntityForWidget();

    $form_display = EntityFormDisplay::create([
      'targetEntityType' => $entity->getEntityTypeId(),
      'bundle' => $entity->bundle(),
      'mode' => 'default',
    ]);

    $form_display->setComponent($field_name);
    $widget = $form_display->getRenderer($field_name);
    $items = $entity->get($field_name);
    $items->filterEmptyItems();
    $entity->set($field_name, $this->configuration['value']);

    $element = ['#parents' => ['field_value']];
    $element += $widget->form($items, $element, $form_state);

    // This custom theme is addressing an issue in core where dialog.ajax is
    // finding the elements in .form-actions and moving them to the bottom of
    // the modal. This causes the "Add another item" button to be moved there
    // as well for entity reference fields.
    // @see Drupal.behaviors.dialog.prepareDialogButtons
    // https://www.drupal.org/project/drupal/issues/3089751
    if (isset($element['widget']['#theme']) && $element['widget']['#theme'] == 'field_multiple_value_form') {
      $element['widget']['#theme'] = 'access_rule_field_multi_value_form';
    }

    return $element;
  }

  /**
   * Extract the submitted field widget values.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array
   *   The submitted field widget values.
   */
  protected function extractFieldWidgetValues(array $form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $field_name = $this->pluginDefinition['field_name'];
    $entity = $this->createStubEntityForWidget();

    // Assign the value to the entity and then retrieve them. This will return
    // properly formatted values.
    $field_values = $values['field_value'][$field_name] ?? [];
    $entity->set($field_name, $field_values);
    $items = $entity->get($field_name);
    $items->filterEmptyItems();
    return $items->getValue();
  }

  /**
   * Create stub entity for the field widget.
   *
   * The field widget requires an entity to interact with. Because we're not
   * actually modifying an entity we need to create a stub.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   A stub entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function createStubEntityForWidget() {
    $field_name = $this->pluginDefinition['field_name'];
    $entity_type_id = reset($this->pluginDefinition['entity_types']);

    $field_map = $this->entityFieldManager->getFieldMap();
    $map = $field_map[$entity_type_id][$field_name] ?? [];

    // Use one of the mapped bundles. It doesn't really matter which one.
    $bundle = reset($map['bundles']);
    $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);
    $bundle_key = $entity_type->getKey('bundle');
    return $this->entityTypeManager->getStorage($entity_type_id)->create([
      $bundle_key => $bundle,
    ]);
  }

  /**
   * Array of applicable entity reference fields found on an entity type.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param \Drupal\Core\Field\FieldDefinitionInterface[] $field_definitions
   *   Array of field definitions.
   * @param array $field_types
   *   Array of field type ids.
   *
   * @return array
   *   The options array.
   */
  private function filterByApplicableFieldStorage($entity_type_id, array $field_definitions, array $field_types) {
    $options = [];
    foreach ($this->entityFieldManager->getFieldStorageDefinitions($entity_type_id) as $field_name => $field_storage) {
      $current_field_type = $field_storage->getType();
      if ($field_storage instanceof FieldStorageConfigInterface
        && !$field_storage->isLocked()
        && in_array($current_field_type, $field_types)) {
        if (isset($field_definitions[$field_name])) {
          $label = $field_definitions[$field_name]->getLabel();
          $options[$field_name] = $label . ': ' . $field_name;
        }
      }
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function validate(EntityInterface $entity, AccountInterface $account) {
    if ($this->configuration['target'] == 'current_user' && empty($this->configuration['user_field'])) {
      return TRUE;
    }

    $entity_values = $this->getEntityFieldValues($entity, $this->pluginDefinition['field_name']);

    if (empty($entity_values)) {
      if ($this->configuration['empty_behavior'] == 'deny') {
        return FALSE;
      }
      else {
        return TRUE;
      }
    }

    // If the content has values then do the comparison.
    $values = $this->getComparisonValues($account);

    // If the user values contains any of the entity values then allow access.
    return $this->validateCallback($entity_values, $values);

  }

  /**
   * Get the values to compare against.
   *
   * @param \Drupal\Core\Session\AccountInterface $user
   *   The current user.
   *
   * @return array
   *   The indexed array of values to compare against.
   */
  protected function getComparisonValues(AccountInterface $user) {
    if ($this->configuration['target'] == 'current_user') {
      return $this->getEntityFieldValues($user, $this->configuration['user_field']);
    }
    elseif ($this->configuration['target'] == 'value') {
      $column_name = $this->getMainPropertyName($this->pluginDefinition['field_type']);
      return array_map(function ($value) use ($column_name) {
        return $value[$column_name];
      }, $this->configuration['value']);
    }
    return [];
  }

  /**
   * Validate method callback.
   *
   * For multi-value fields, validation callbacks operate on each value. This
   * is to align with how queries match content with conditions.
   *
   * @param array $entity_values
   *   Array of entity values.
   * @param array $values
   *   The user values.
   *
   * @return bool
   *   TRUE if valid; FALSE otherwise.
   */
  public function validateCallback(array $entity_values, array $values) {
    $operator = $this->configuration['operator'];
    $info = $this->operators();
    if (!empty($info[$operator]['method'])) {
      // If any of the field values match then return true.
      foreach ($entity_values as $value) {
        $valid = $this->{$info[$operator]['method']}($value, $values);
        if ($valid) {
          return TRUE;
        }
      }
    }

    return FALSE;
  }

}
