<?php

namespace Drupal\access_policy\Plugin\AccessRule;

/**
 * Restrict content by comparing field values.
 *
 * @AccessRule(
 *   id = "field_value_string",
 *   base_plugin_id = "field_value",
 *   field_types = {
 *     "string",
 *     "string_long",
 *   },
 * )
 */
class FieldValueString extends FieldValueBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      "operator" => '=',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function operators() {
    $operators = [
      '=' => [
        'title' => $this->t('Is equal to'),
        'method' => 'validateSimple',
      ],
      '!=' => [
        'title' => $this->t('Is not equal to'),
        'method' => 'validateSimple',
      ],
      'contains' => [
        'title' => $this->t('Contains'),
        'method' => 'validateContains',
      ],
      'not contains' => [
        'title' => $this->t('Does not contain'),
        'method' => 'validateContains',
      ],
      'starts with' => [
        'title' => $this->t('Starts with'),
        'method' => 'validateStartsWith',
      ],
      'not starts with' => [
        'title' => $this->t('Does not start with'),
        'method' => 'validateStartsWith',
      ],
      'ends with' => [
        'title' => $this->t('Ends with'),
        'method' => 'validateEndsWith',
      ],
      'not ends with' => [
        'title' => $this->t('Does not end with'),
        'method' => 'validateEndsWith',
      ],
    ];

    return $operators;
  }

  /**
   * Validate operators.
   *
   * @param mixed $value
   *   The entity value.
   * @param array $options
   *   The value options.
   *
   * @return bool
   *   TRUE if operator passes; FALSE otherwise.
   */
  public function validateSimple($value, array $options) {
    $op = $this->configuration['operator'];
    switch ($op) {
      case '=':
        $equals = $this->equals($value, $options);
        if ($equals) {
          return TRUE;
        }
        break;

      case '!=':
        $equals = $this->equals($value, $options);
        if (!$equals) {
          return TRUE;
        }
    }

    return FALSE;
  }

  /**
   * Validate contains and not contains operators.
   *
   * @param mixed $value
   *   The entity value.
   * @param array $options
   *   The value options.
   *
   * @return bool
   *   TRUE if operator passes; FALSE otherwise.
   */
  public function validateContains($value, array $options) {
    $op = $this->configuration['operator'];
    switch ($op) {
      case 'contains':
        $contains = $this->contains($value, $options);
        if ($contains) {
          return TRUE;
        }
        break;

      case 'not contains':
        $contains = $this->contains($value, $options);
        if (!$contains) {
          return TRUE;
        }
        break;
    }

    return FALSE;
  }

  /**
   * Contains operator.
   *
   * @param mixed $value
   *   The current value.
   * @param array $options
   *   Array of options to compare against.
   *
   * @return bool
   *   TRUE if the value has values in $options; FALSE otherwise.
   */
  public function contains($value, array $options) {
    foreach ($options as $option) {
      if (substr_count($value, $option) > 0) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Validate starts with and not starts with operators.
   *
   * @param mixed $value
   *   The entity value.
   * @param array $options
   *   The value options.
   *
   * @return bool
   *   TRUE if operator passes; FALSE otherwise.
   */
  public function validateStartsWith($value, array $options) {
    $op = $this->configuration['operator'];

    switch ($op) {
      case 'starts with':
        foreach ($options as $option) {
          $length = strlen($option);
          if (substr($value, 0, $length) == $option) {
            return TRUE;
          }
        }
        break;

      case 'not starts with':
        foreach ($options as $option) {
          $length = strlen($option);
          if (substr($value, 0, $length) != $option) {
            return TRUE;
          }
        }
        break;
    }

    return FALSE;
  }

  /**
   * Validate starts with and not starts with operators.
   *
   * @param mixed $value
   *   The entity value.
   * @param array $options
   *   The value options.
   *
   * @return bool
   *   TRUE if operator passes; FALSE otherwise.
   */
  public function validateEndsWith($value, array $options) {
    $op = $this->configuration['operator'];

    switch ($op) {
      case 'ends with':
        foreach ($options as $option) {
          $offset = strlen($option);
          if (substr($value, -$offset) == $option) {
            return TRUE;
          }
        }
        break;

      case 'not ends with':
        foreach ($options as $option) {
          $offset = strlen($option);
          if (substr($value, -$offset) != $option) {
            return TRUE;
          }
        }
        break;
    }

    return FALSE;
  }

}
