<?php

namespace Drupal\access_policy\Plugin\AccessRule;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Grant access to users if referenced in an entity reference field.
 *
 * @AccessRule(
 *   id = "user_entity_reference",
 *   label = @Translation("Is referenced by current user"),
 *   description = @Translation("Grant access if referenced by a field on the current user."),
 *   handlers = {
 *     "query_alter" = "\Drupal\access_policy\AccessRuleQueryHandler\UserEntityReference"
 *   }
 * )
 */
class UserEntityReference extends AccessRuleBase implements AccessRuleFieldInterface {

  use AccessRuleOperatorTrait;
  use AccessRuleFieldTrait;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $fieldTypeManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a UserReference access rule object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_manager
   *   The field type plugin manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityFieldManagerInterface $entity_field_manager, FieldTypePluginManagerInterface $field_type_manager, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityFieldManager = $entity_field_manager;
    $this->fieldTypeManager = $field_type_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function isApplicable(EntityInterface $entity) {
    $config = $this->getConfiguration();
    if (isset($config['entity_field'])) {
      $user = $this->entityTypeManager->getStorage('user')->load(\Drupal::currentUser()->id());
      $entity_field = $config['entity_field'];
      if ($user->hasField($entity_field)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      "entity_field" => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function accessRuleForm(array $form, FormStateInterface $form_state) {
    $entity_type = $this->getAccessPolicy()->getTargetEntityType();
    $form['entity_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Entity reference'),
      '#options' => $this->getUserFieldOptions(),
      '#default_value' => $this->configuration['entity_field'],
      '#empty_option' => $this->t('- Select -'),
      '#description' => $this->t('Users will be granted access to any :entity_type referenced in this field.', [':entity_type' => $entity_type->getCollectionLabel()]),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function accessRuleFormSubmit(array &$form, FormStateInterface $form_state) {
    $this->configuration['entity_field'] = $form_state->getValue('entity_field');
  }

  /**
   * Get options array for entity field.
   *
   * @return array
   *   The options array.
   */
  private function getUserFieldOptions() {
    $entity_type_id = $this->getAccessPolicy()->getTargetEntityTypeId();

    $options = [];
    $field_definitions = $this->entityFieldManager->getFieldDefinitions('user', 'user');
    foreach ($field_definitions as $definition) {
      if ($definition->getType() == 'entity_reference' && $definition->getSetting('handler') == 'default:' . $entity_type_id) {
        $options[$definition->getName()] = $definition->label() . ': ' . $definition->getName();
      }
    }

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function validate(EntityInterface $entity, AccountInterface $account) {
    if (empty($this->configuration['entity_field'])) {
      return TRUE;
    }

    $entity_field = $this->configuration['entity_field'];
    // If it's a multi-value field then compare all the values. If any of them
    // match then return true.
    $entity_values = array_map(function ($field) {
      if (isset($field['target_id'])) {
        return $field['target_id'];
      }
    }, $account->get($entity_field)->getValue());

    if (in_array($entity->id(), $entity_values)) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldName() {
    return $this->configuration['entity_field'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFieldType() {
    return 'entity_reference';
  }

}
