<?php

namespace Drupal\access_policy\Plugin\AccessRuleWidget;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Embedded entity field access rule widget.
 *
 * Renders  the entity field on the access tab.
 *
 * @AccessRuleWidget(
 *   id = "entity_field",
 *   label = @Translation("Entity field"),
 *   description = @Translation("Shows the field on the access tab.")
 * )
 */
class EntityField extends AccessRuleWidgetBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    // Get the widget options based on field type.
    $field_type = $this->getFieldType();
    $widgetPluginManager = \Drupal::service('plugin.manager.field.widget');
    $options = $widgetPluginManager->getOptions($field_type);

    $form['field_widget'] = [
      '#title' => $this->t('Field widget display'),
      '#type' => 'select',
      '#default_value' => $this->getWidgetSetting('field_widget'),
      '#options' => $options,
    ];
    return $form;
  }

  /**
   * Get the field name from the access rule plugin.
   *
   * @return string|void
   *   The source field name or NULL if not configured.
   */
  private function getFieldName() {
    return $this->definition->getFieldName();
  }

  /**
   * Get the field type from the access rule plugin.
   *
   * @return string
   *   The field type.
   */
  private function getFieldType() {
    return $this->definition->getFieldType();
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm($form, $form_state) {
    $field_name = $this->getFieldName();
    $entity = $this->getEntity();

    $new_form_display = EntityFormDisplay::create([
      'targetEntityType' => $entity->getEntityTypeId(),
      'bundle' => $entity->bundle(),
      'mode' => 'default',
    ]);

    $options = [];
    if (!empty($this->getWidgetSetting('field_widget'))) {
      $options['type'] = $this->getWidgetSetting('field_widget');
    }
    $new_form_display->setComponent($field_name, $options);

    $widget = $new_form_display->getRenderer($field_name);
    $items = $entity->get($field_name);
    $items->filterEmptyItems();

    $new_form_display->buildForm($entity, $form, $form_state);
    $widget_form = $widget->form($items, $form, $form_state);

    if ($this->isNotTranslatable($entity, $field_name)) {
      $url = $entity->getUntranslated()->toUrl('access-policy-form')->toString();
      \Drupal::messenger()->addWarning($this->t('Fields that apply to all languages are disabled to avoid conflicting changes. <a href=":url">Edit them on the original language access form</a>.', [':url' => $url]));
      $widget_form['widget']['#disabled'] = TRUE;
    }

    return $widget_form;
  }

  /**
   * Determine whether the field is translateable.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   * @param string $field_name
   *   The current field.
   *
   * @return bool
   *   TRUE if not translateable; FALSE otherwise.
   */
  private function isNotTranslatable(EntityInterface $entity, $field_name) {
    $items = $entity->get($field_name);
    $definition = $items->getFieldDefinition();
    if (!$entity->language()->isDefault() && !$definition->isTranslatable()) {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $field_name = $this->getFieldName();
    $value = $form_state->getValue($field_name);

    if (is_array($value)) {
      // This was probably a multi-value field, remove any non-numeric keys.
      foreach ($value as $key => $element) {
        if (!is_numeric($key)) {
          unset($value[$key]);
        }
      }
    }

    $entity = $this->getEntity();
    $entity->set($field_name, $value);
    $entity->save();
  }

}
