<?php

namespace Drupal\access_policy\Plugin\Derivative;

use Drupal\access_policy\AccessPolicyInformation;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\field\FieldStorageConfigInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides field value access rule plugin definitions.
 */
class FieldValueAccessRule implements ContainerDeriverInterface {

  use StringTranslationTrait;

  /**
   * Stores all entity row plugin information.
   *
   * @var array
   */
  protected $derivatives = [];

  /**
   * The base plugin ID that the derivative is for.
   *
   * @var string
   */
  protected $basePluginId;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Bundle information service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfo
   */
  protected $bundleInfo;

  /**
   * Access policy information service.
   *
   * @var \Drupal\access_policy\AccessPolicyInformation
   */
  protected $accessPolicyInfo;

  /**
   * Array of excluded field types.
   *
   * These field types are not currently supported. When new access rule plugins
   * are added this will change.
   *
   * @var array
   *   Array of excluded field types.
   */
  protected $excludedFieldTypes = [
    'comment',
    'image',
    'file',
    'link',
    'timestamp',
  ];

  /**
   * Constructs a ViewsEntityRow object.
   *
   * @param string $base_plugin_id
   *   The base plugin ID.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfo $bundle_info
   *   Bundle information service.
   * @param \Drupal\access_policy\AccessPolicyInformation $access_policy_info
   *   Access policy information service.
   */
  public function __construct($base_plugin_id, EntityTypeManagerInterface $entity_type_manager, EntityFieldManagerInterface $entity_field_manager, EntityTypeBundleInfo $bundle_info, AccessPolicyInformation $access_policy_info) {
    $this->basePluginId = $base_plugin_id;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->bundleInfo = $bundle_info;
    $this->accessPolicyInfo = $access_policy_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $base_plugin_id,
      $container->get('entity_type.manager'),
      $container->get('entity_field.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('access_policy.information'),

    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinition($derivative_id, $base_plugin_definition) {
    if (!empty($this->derivatives) && !empty($this->derivatives[$derivative_id])) {
      return $this->derivatives[$derivative_id];
    }
    $this->getDerivativeDefinitions($base_plugin_definition);
    return $this->derivatives[$derivative_id];
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type) {
      if ($this->accessPolicyInfo->isAccessControlledEntityType($entity_type)) {
        $bundles = $this->bundleInfo->getBundleInfo($entity_type->id());
        $bundle_options = [];
        foreach ($bundles as $bundle_id => $bundle) {
          $bundle_options[] = $bundle_id;
        }
        $field_definitions = $this->getFieldDefinitions($entity_type, $bundle_options);
        $fields = $this->filterByApplicableFieldStorage($entity_type->id(), $field_definitions);

        if (!empty($fields)) {
          foreach ($fields as $field_name => $definition) {
            $this->derivatives[$entity_type->id() . ':' . $field_name] = [
              'id' => $entity_type->id() . ':' . $field_name,
              'provider' => 'access_policy',
              'label' => $this->t('Field value: @label', [
                '@label' => $definition->getLabel(),
              ]),
              'description' => $this->t('Control access by comparing values in this field.'),
              'access_rule_widget' => 'entity_field',
              'handlers' => [
                'query_alter' => '\Drupal\access_policy\AccessRuleQueryHandler\FieldValue',
              ],
              'entity_types' => [$definition->getTargetEntityTypeId()],
              'field_type' => $definition->getType(),
              'field_name' => $field_name,
              'class' => $base_plugin_definition['class'],
            ];
          }
        }
      }
    }

    return $this->derivatives;
  }

  /**
   * Aggregate of field definitions for supported content entity types/bundles.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type.
   * @param array $bundles
   *   The array of bundles.
   *
   * @return \Drupal\Core\Field\FieldDefinitionInterface[]
   *   Array of field definitions.
   */
  private function getFieldDefinitions(EntityTypeInterface $entity_type, array $bundles) {
    $field_definitions = [];
    foreach ($bundles as $bundle) {
      $field_definitions = array_merge($field_definitions, $this->entityFieldManager->getFieldDefinitions($entity_type->id(), $bundle));
    }

    return $field_definitions;
  }

  /**
   * Array of applicable entity reference fields found on an entity type.
   *
   * @param string $entity_type_id
   *   The entity type id.
   * @param \Drupal\Core\Field\FieldDefinitionInterface[] $field_definitions
   *   Array of field definitions.
   *
   * @return \Drupal\Core\Field\FieldDefinitionInterface[]
   *   Array of filtered field definitions.
   */
  private function filterByApplicableFieldStorage($entity_type_id, array $field_definitions) {
    $options = [];
    foreach ($this->entityFieldManager->getFieldStorageDefinitions($entity_type_id) as $field_name => $field_storage) {
      if ($field_storage instanceof FieldStorageConfigInterface
        && !$this->isExcluded($field_storage->getType())
        && !$field_storage->isLocked()) {
        if (isset($field_definitions[$field_name])) {
          $options[$field_name] = $field_definitions[$field_name];
        }
      }
    }
    return $options;
  }

  /**
   * Determine whether a particular field type is excluded.
   *
   * @param string $field_type
   *   The field type.
   *
   * @return bool
   *   TRUE if the field type is excluded; FALSE otherwise.
   */
  private function isExcluded($field_type) {
    if (in_array($field_type, $this->excludedFieldTypes)) {
      return TRUE;
    }

    return FALSE;
  }

}
