<?php

namespace Drupal\Tests\access_policy\Functional;

use Drupal\access_policy\Entity\AccessPolicy;

/**
 * Tests the creation and deletion of access policy entities.
 *
 * @group access_policy
 */
class AccessPolicyCreationTest extends AccessPolicyTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'access_policy',
    'access_policy_test',
    'filter',
    'node',
    'datetime',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Set to TRUE to strict check all configuration saved.
   *
   * @var bool
   *
   * @see \Drupal\Core\Config\Development\ConfigSchemaChecker
   */
  protected $strictConfigSchema = FALSE;

  /**
   * Tests creating and deleting a new access policy with rules and bundles.
   */
  public function testCreateAndDeleteAccessPolicyWithRules() {

    $rid = $this->drupalCreateRole([
      'access administration pages',
      'administer access policy entities',
    ]);
    $web_user = $this->drupalCreateUser();
    $web_user->addRole($rid);
    $web_user->save();

    $this->drupalLogin($web_user);

    $this->drupalGet('admin/people/access_policy');
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/people/access_policy/add');

    $edit = [
      'label' => 'Foo',
      'id' => 'foo',
      'type' => 'group',
    ];
    $this->submitForm($edit, 'Save');

    // Confirm that permissions warning appears on new policies.
    $this->assertSession()->pageTextContains('This access policy uses permissions.');

    // Configure the bundles.
    $this->drupalGet('admin/people/access_policy/foo/bundles');
    $this->assertSession()->statusCodeEquals(200);

    $edit = [
      'bundles[page]' => TRUE,
    ];
    $this->submitForm($edit, 'Save');

    $this->drupalGet('/admin/people/access_policy/foo');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Page');

    // Add a rule to the policy (cancel)
    $this->drupalGet('/admin/people/access_policy/foo/rule/add');
    $this->assertSession()->statusCodeEquals(200);
    $edit = [
      'access_rules[is_own]' => TRUE,
    ];
    $this->submitForm($edit, 'Add rule');
    // Go to access rule configuration form and press cancel. Make sure it
    // does not save anything.
    $this->submitForm([], 'Cancel');
    $this->drupalGet('/admin/people/access_policy/foo');
    $this->assertSession()->pageTextNotContains('Authored by current user');

    // Add a rule to the policy (submit).
    $this->drupalGet('/admin/people/access_policy/foo/rule/add');
    $this->assertSession()->statusCodeEquals(200);
    $edit = [
      'access_rules[is_own]' => TRUE,
    ];
    $this->submitForm($edit, 'Add rule');
    $this->assertSession()->fieldValueEquals('query', TRUE);
    $this->assertSession()->fieldValueEquals('admin_label', '');
    $this->submitForm([], 'Save');

    $this->drupalGet('/admin/people/access_policy/foo');
    $this->assertSession()->pageTextContains('Authored by current user');

    // Change settings of is_own access rule.
    $this->drupalGet('/admin/people/access_policy/foo/rule/is_own');
    $this->assertSession()->statusCodeEquals(200);
    $edit = [
      'query' => FALSE,
      'admin_label' => 'Authored by current user (modified)',
    ];
    $this->submitForm($edit, 'Save');

    $this->drupalGet('/admin/people/access_policy/foo');
    $this->assertSession()->pageTextContains('Authored by current user (modified)');

    $this->drupalGet('/admin/people/access_policy');
    $this->assertSession()->pageTextContains('Authored by current user (modified)');

    $this->drupalGet('/admin/people/access_policy/foo/rule/is_own');
    $this->assertSession()->fieldValueEquals('query', FALSE);
    $this->assertSession()->fieldValueEquals('admin_label', 'Authored by current user (modified)');

    // Delete the is_own access rule.
    $this->drupalGet('/admin/people/access_policy/foo/rule/is_own/remove');
    $this->assertSession()->pageTextContains('Are you sure you want to remove this rule?');
    $this->submitForm([], 'Remove');
    $this->drupalGet('/admin/people/access_policy/foo');
    $this->assertSession()->pageTextNotContains('Authored by current user (modified)');

    // Change the other fields in the policy.
    $edit = [
      'label' => 'Bar',
      'description' => 'Lorem ipsum dolor.',
      'access_rule_operator' => 'OR',
    ];
    $this->submitForm($edit, 'Save');

    $this->clickLink('Configure');

    $this->assertSession()->fieldValueEquals('label', 'Bar');
    $this->assertSession()->fieldValueEquals('description', 'Lorem ipsum dolor.');
    $this->assertSession()->fieldValueEquals('access_rule_operator', 'OR');

    $this->clickLink('Delete');
    $this->assertSession()->pageTextContains('Are you sure you want to delete Bar?');
    $this->submitForm([], 'Delete');
    $this->assertSession()->pageTextContains('Deleted Bar Access Policy');

    $policy = $this->entityTypeManager->getStorage('access_policy')->load('foo');
    $this->assertEmpty($policy, 'The policy should have been deleted.');
  }

  /**
   * Tests setting creating a new access policy.
   */
  public function testDoNotPermitPolicyDeletionWhenUsed() {
    AccessPolicy::create([
      'id' => 'test_policy',
      'label' => 'Test policy',
      'access_rules' => [],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ])->save();

    $node = $this->drupalCreateNode([
      'type' => 'page',
    ]);
    $this->assignAccessPolicy($node, 'test_policy');

    $web_user = $this->drupalCreateUser([
      'access administration pages',
      'administer access policy entities',
    ]);
    $this->drupalLogin($web_user);

    $this->drupalGet('admin/people/access_policy/test_policy/delete');

    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Are you sure you want to delete Test policy?');
    $this->assertSession()->pageTextContains('The access policy Test policy is being used, and cannot be deleted.');
    $this->assertSession()->linkNotExists('Delete');
  }

  /**
   * Tests creating an access policy with automatic entity selection.
   */
  public function testCreateWithAutomaticEntitySelection() {
    // The new policy will inherit the entity types settings of this policy.
    AccessPolicy::create([
      'id' => 'test_policy',
      'label' => 'Test policy',
      'access_rules' => [],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ])->save();

    $web_user = $this->drupalCreateUser([
      'access administration pages',
      'administer access policy entities',
    ]);
    $this->drupalLogin($web_user);

    $this->drupalGet('admin/people/access_policy');
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/people/access_policy/add');

    $edit = [
      'label' => 'Foo',
      'id' => 'foo',
      'type' => 'private',
    ];
    $this->submitForm($edit, 'Save');

    $this->drupalGet('/admin/people/access_policy/foo');
    $edit = [
      'bundles_container[entity_type_auto_select]' => TRUE,
    ];
    $this->submitForm($edit, 'Save');

    $policy = $this->entityTypeManager->getStorage('access_policy')->load('foo');
    $this->assertEquals(TRUE, $policy->isAutomaticEntityTypeSelection(), 'The entity type selection should be automatic.');
    $expected = ['page'];
    $this->assertEquals($expected, $policy->getTargetBundles(), 'The entity types did not match');

    $this->drupalGet('/admin/people/access_policy/foo');
    $this->assertSession()->fieldValueEquals('bundles_container[entity_type_auto_select]', TRUE);

    // Update the parent policy and make sure that it propagates to the
    // automatic policies.
    $this->drupalCreateContentType([
      'type' => 'article',
      'name' => 'Article',
    ]);

    $this->drupalGet('/admin/people/access_policy/test_policy/bundles');
    $this->assertSession()->statusCodeEquals(200);
    $edit = [
      'bundles[article]' => TRUE,
      'bundles[page]' => TRUE,
    ];
    $this->submitForm($edit, 'Save');

    $policy = $this->entityTypeManager->getStorage('access_policy')->load('foo');
    $expected = ['article', 'page'];
    $this->assertEquals($expected, $policy->getTargetBundles(), 'The entity types did not match');

    // Remove all the bundles and make sure it propagates to the automatic
    // policies.
    $this->drupalGet('/admin/people/access_policy/test_policy/bundles');
    $edit = [
      'bundles[article]' => FALSE,
      'bundles[page]' => FALSE,
    ];
    $this->submitForm($edit, 'Save');
    $policy = $this->entityTypeManager->getStorage('access_policy')->load('foo');

    $expected = [];
    $this->assertEquals($expected, $policy->getTargetBundles(), 'No entity types should be selected.');

    // Also make sure that programmatically updating any access policy also
    // updates the automatic policy.
    $test_policy = $this->entityTypeManager->getStorage('access_policy')->load('test_policy');
    $test_policy->setTargetBundles(['page']);
    $test_policy->save();

    $policy = $this->entityTypeManager->getStorage('access_policy')->load('foo');
    $expected = ['page'];
    $this->assertEquals($expected, $policy->getTargetBundles(), 'The entity types did not match');
  }

  /**
   * Tests creating a private access policy.
   */
  public function testCreatePrivateAccessPolicy() {

    $web_user = $this->drupalCreateUser([
      'access administration pages',
      'administer access policy entities',
    ]);
    $this->drupalLogin($web_user);

    $this->drupalGet('admin/people/access_policy');
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/people/access_policy/add');

    $edit = [
      'label' => 'Foo',
      'id' => 'foo',
      'type' => 'private',
    ];
    $this->submitForm($edit, 'Save');

    $this->assertSession()->pageTextContains('Authored by current user');

    // Confirm that permissions warning does not appear because private
    // policies do not support permissions.
    $this->assertSession()->pageTextNotContains('This access policy uses permissions.');

    // Make sure that the delete link is not visible nor accessible.
    $this->assertSession()->linkByHrefNotExists('admin/people/access_policy/foo/rule/private/remove');
    $this->drupalGet('admin/people/access_policy/foo/rule/private/remove');
    $this->assertSession()->statusCodeEquals(403);
  }

  /**
   * Tests creating a new public access policy.
   */
  public function testCreatePublicPolicy() {

    $web_user = $this->drupalCreateUser([
      'access administration pages',
      'administer access policy entities',
    ]);
    $this->drupalLogin($web_user);

    $this->drupalGet('admin/people/access_policy');
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/people/access_policy/add');

    $edit = [
      'label' => 'Foo',
      'id' => 'foo',
      'type' => 'public',
    ];
    $this->submitForm($edit, 'Save');

    $this->assertSession()->pageTextNotContains('Access Rules');
    $this->assertSession()->linkNotExists('Add Rule');
    $this->assertSession()->linkNotExists('Validation operator');
  }

}
