<?php

namespace Drupal\Tests\access_policy\Functional;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\Tests\field\Traits\EntityReferenceTestTrait;
use Drupal\access_policy\Entity\AccessPolicy;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * Ensures that listings only display entities that a user is allowed to view.
 *
 * @group access_policy
 */
class AccessRuleFieldValueTest extends AccessPolicyTestBase {

  use UserCreationTrait;
  use EntityReferenceTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'access_policy',
    'access_policy_test',
    'filter',
    'node',
    'field',
    'user',
    'datetime',
    'views',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Set to TRUE to strict check all configuration saved.
   *
   * @var bool
   *
   * @see \Drupal\Core\Config\Development\ConfigSchemaChecker
   */
  protected $strictConfigSchema = FALSE;

  /**
   * Tests viewing a node with a field value rule comparing values with the
   * current user.
   */
  public function testFieldValueTextFieldWithCurrentUser() {
    AccessPolicy::create([
      'id' => 'matching_string',
      'label' => 'Matching String Access Policy',
      'access_rules' => [
        'field_value:node:field_text' => [
          'plugin_id' => 'field_value:node:field_text',
          'settings' => [
            'target' => 'current_user',
            'user_field' => 'field_text',
            'operator' => '=',
            'query' => TRUE,
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ])->save();

    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1 (With text: Foo)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Foo',
      ],
    ]);

    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2 (With text: Bar)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Bar',
      ],
    ]);

    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3 (Without text)',
      'type' => 'page',
      'status' => 1,
    ]);

    $this->assignAccessPolicy($node_1, 'matching_string');
    $this->assignAccessPolicy($node_2, 'matching_string');
    $this->assignAccessPolicy($node_3, 'matching_string');

    // Login as a user with a matching entity reference. They should be granted
    // access.
    $admin_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view matching_string content',
    ]);
    $admin_user->set('field_text', [
      'Foo',
    ]);
    $admin_user->save();

    $this->drupalLogin($admin_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1 (With text: Foo)');
    $this->assertSession()->pageTextNotContains('Node 2 (With text: Bar)');
    $this->assertSession()->pageTextContainsOnce('Node 3 (Without text)');

    // Create a user with no string field values. They should only be able to
    // see content without a string value.
    $web_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view matching_string content',
    ]);
    $this->drupalLogin($web_user);

    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains('Node 1 (With text: Foo)');
    $this->assertSession()->pageTextNotContains('Node 2 (With text: Bar)');
    $this->assertSession()->pageTextContainsOnce('Node 3 (Without text)');
  }

  /**
   * Tests viewing a node that has hardcoded values in the access rule.
   */
  public function testFieldValueTextFieldWithValue() {
    AccessPolicy::create([
      'id' => 'matching_string',
      'label' => 'Matching String Access Policy',
      'access_rules' => [
        'field_value:node:field_text' => [
          'plugin_id' => 'field_value:node:field_text',
          'settings' => [
            'target' => 'value',
            'value' => [
              ['value' => 'Foo'],
            ],
            'operator' => '=',
            'query' => TRUE,
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ])->save();

    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1 (With text: Foo)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Foo',
      ],
    ]);

    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2 (With text: Bar)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Bar',
      ],
    ]);

    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3 (Without text)',
      'type' => 'page',
      'status' => 1,
    ]);

    $this->assignAccessPolicy($node_1, 'matching_string');
    $this->assignAccessPolicy($node_2, 'matching_string');
    $this->assignAccessPolicy($node_3, 'matching_string');

    // We're not comparing field values with a user so it's not necessary to set
    // it.
    $admin_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view matching_string content',
    ]);
    $admin_user->save();

    $this->drupalLogin($admin_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1 (With text: Foo)');
    $this->assertSession()->pageTextNotContains('Node 2 (With text: Bar)');
    $this->assertSession()->pageTextContainsOnce('Node 3 (Without text)');
  }

  /**
   * Tests the field value access rule with entity reference fields.
   */
  public function testFieldValueEntityReferenceWithCurrentUser() {
    AccessPolicy::create([
      'id' => 'match_reference_user',
      'label' => 'Matching entity reference Access Policy',
      'access_rules' => [
        'field_value:node:field_entity_reference' => [
          'plugin_id' => 'field_value:node:field_entity_reference',
          'settings' => [
            'target' => 'current_user',
            'user_field' => 'field_entity_reference',
            'operator' => 'in',
            'query' => TRUE,
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ])->save();

    $this->createEntityReferenceField('node', 'page', 'field_entity_reference', 'Entity Reference', 'node', 'default', [], -1);
    $this->createEntityReferenceField('user', 'user', 'field_entity_reference', 'Entity Reference', 'node', 'default', [], -1);

    $referenced_node_1 = $this->drupalCreateNode([
      'title' => 'Referenced Node #1',
      'type' => 'page',
      'status' => 1,
    ]);
    $referenced_node_2 = $this->drupalCreateNode([
      'title' => 'Referenced Node #2',
      'type' => 'page',
      'status' => 1,
    ]);
    $referenced_node_3 = $this->drupalCreateNode([
      'title' => 'Referenced Node #3',
      'type' => 'page',
      'status' => 1,
    ]);

    $web_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view match_reference_user content',
    ]);
    $web_user->set('field_entity_reference', [
      $referenced_node_1,
    ]);
    $web_user->save();

    // Adding two options in order to test for duplicates in the join.
    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
      'field_entity_reference' => [
        $referenced_node_1,
        $referenced_node_2,
      ],
    ]);

    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2',
      'type' => 'page',
      'status' => 1,
      'field_entity_reference' => [
        $referenced_node_3,
      ],
    ]);

    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3',
      'type' => 'page',
      'status' => 1,
    ]);

    $this->assignAccessPolicy($node_1, 'match_reference_user');
    $this->assignAccessPolicy($node_2, 'match_reference_user');
    $this->assignAccessPolicy($node_3, 'match_reference_user');

    // Login as a user with a matching entity reference. They should be granted
    // access.
    $this->drupalLogin($web_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1');
    $this->assertSession()->pageTextNotContains('Node 2');
    $this->assertSession()->pageTextContainsOnce('Node 3');
  }

  /**
   * Tests the field value access rule with entity reference fields.
   */
  public function testFieldValueEntityReferenceWithEmptyCurrentUserDenied() {
    AccessPolicy::create([
      'id' => 'match_reference_user',
      'label' => 'Matching entity reference Access Policy',
      'access_rules' => [
        'field_value:node:field_entity_reference' => [
          'plugin_id' => 'field_value:node:field_entity_reference',
          'settings' => [
            'target' => 'current_user',
            'user_field' => 'field_entity_reference',
            'operator' => 'in',
            'query' => TRUE,
            'empty_behavior' => 'deny',
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ])->save();

    $this->createEntityReferenceField('node', 'page', 'field_entity_reference', 'Entity Reference', 'node', 'default', [], -1);
    $this->createEntityReferenceField('user', 'user', 'field_entity_reference', 'Entity Reference', 'node', 'default', [], -1);

    $referenced_node_1 = $this->drupalCreateNode([
      'title' => 'Referenced Node #1',
      'type' => 'page',
      'status' => 1,
    ]);

    $web_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view match_reference_user content',
    ]);
    $web_user->set('field_entity_reference', [
      $referenced_node_1,
    ]);
    $web_user->save();

    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
    ]);

    $this->assignAccessPolicy($node_1, 'match_reference_user');

    // Login as a user with an entity reference, however the node does not have
    // an entity reference and the empty_behavior is set to deny. They should
    // not be able to see the content.
    $this->drupalLogin($web_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains('Node 1');

    // This user doesn't have an entity reference, they also shouldn't be able
    // to see the content.
    $web_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view match_reference_user content',
    ]);

    $this->drupalLogin($web_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains('Node 1');
  }

  /**
   * Tests the field value access rule with entity reference fields.
   */
  public function testFieldValueEntityReferenceWithValue() {

    $policy = AccessPolicy::create([
      'id' => 'match_reference_value',
      'label' => 'Matching entity reference with value',
      'access_rules' => [
        'field_value:node:field_entity_reference' => [
          'plugin_id' => 'field_value:node:field_entity_reference',
          'settings' => [
            'target' => 'value',
            'value' => [
              ['target_id' => 1],
            ],
            'operator' => 'in',
            'query' => TRUE,
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ]);

    $policy->save();

    $this->createEntityReferenceField('node', 'page', 'field_entity_reference', 'Entity Reference', 'node', 'default', [], -1);

    $referenced_node_1 = $this->drupalCreateNode([
      'nid' => 1,
      'title' => 'Referenced Node #1',
      'type' => 'page',
      'status' => 1,
    ]);

    $referenced_node_2 = $this->drupalCreateNode([
      'title' => 'Referenced Node #1',
      'type' => 'page',
      'status' => 1,
    ]);

    // This is referencing node 1 in the access policy and should be visible.
    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
      'field_entity_reference' => [
        $referenced_node_1,
      ],
    ]);

    // This is referencing a different node and shoud not be visibe.
    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2',
      'type' => 'page',
      'status' => 1,
      'field_entity_reference' => [
        $referenced_node_2,
      ],
    ]);

    // This has no value and will therefore be visible.
    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3',
      'type' => 'page',
      'status' => 1,
    ]);

    $this->assignAccessPolicy($node_1, 'match_reference_value');
    $this->assignAccessPolicy($node_2, 'match_reference_value');
    $this->assignAccessPolicy($node_3, 'match_reference_value');

    $web_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view match_reference_value content',
    ]);

    // We're not matching values on the user, so we don't need to assign it.
    $this->drupalLogin($web_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1');
    $this->assertSession()->pageTextNotContains('Node 2');
    $this->assertSession()->pageTextContains('Node 3');

  }

  /**
   * Tests the field value access rule with entity reference fields.
   */
  public function testFieldValueEntityReferenceWithEmptyValueDenied() {
    $policy = AccessPolicy::create([
      'id' => 'match_reference_value',
      'label' => 'Matching entity reference with value',
      'access_rules' => [
        'field_value:node:field_entity_reference' => [
          'plugin_id' => 'field_value:node:field_entity_reference',
          'settings' => [
            'target' => 'value',
            'value' => [
              ['target_id' => 1],
            ],
            'operator' => 'in',
            'query' => TRUE,
            'empty_behavior' => 'deny',
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ]);

    $policy->save();

    $this->createEntityReferenceField('node', 'page', 'field_entity_reference', 'Entity Reference', 'node', 'default', [], -1);

    // This has no value and will therefore be visible.
    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3',
      'type' => 'page',
      'status' => 1,
    ]);

    $this->assignAccessPolicy($node_3, 'match_reference_value');

    $web_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view match_reference_value content',
    ]);

    // We're not matching values on the user, so we don't need to assign it.
    $this->drupalLogin($web_user);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains('Node 3');
  }

  /**
   * Tests the field value access rule with not in operator.
   */
  public function testFieldValueEntityReferenceWithNotInOperator() {
    $policy = AccessPolicy::create([
      'id' => 'match_reference_value',
      'label' => 'Matching entity reference with value',
      'access_rules' => [
        'field_value:node:field_entity_reference' => [
          'plugin_id' => 'field_value:node:field_entity_reference',
          'settings' => [
            'target' => 'value',
            'value' => [
              ['target_id' => 1],
            ],
            'operator' => 'not in',
            'query' => TRUE,
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ]);

    $policy->save();

    $this->createEntityReferenceField('node', 'page', 'field_entity_reference', 'Entity Reference', 'node', 'default', [], -1);

    $referenced_node_1 = $this->drupalCreateNode([
      'nid' => 1,
      'title' => 'Referenced Node #1',
      'type' => 'page',
      'status' => 1,
    ]);

    $referenced_node_2 = $this->drupalCreateNode([
      'title' => 'Referenced Node #1',
      'type' => 'page',
      'status' => 1,
    ]);

    // This is referencing node 1 in the access policy and should not be
    // visible.
    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
      'field_entity_reference' => [
        $referenced_node_1,
      ],
    ]);

    // This is referencing a different node and should be visible.
    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2',
      'type' => 'page',
      'status' => 1,
      'field_entity_reference' => [
        $referenced_node_2,
      ],
    ]);

    $this->assignAccessPolicy($node_1, 'match_reference_value');
    $this->assignAccessPolicy($node_2, 'match_reference_value');

    $web_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view match_reference_value content',
    ]);

    // We're not matching values on the user, so we don't need to assign it.
    $this->drupalLogin($web_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains('Node 1');
    $this->assertSession()->pageTextContainsOnce('Node 2');
  }

  /**
   * Tests a node with numeric field values between values.
   */
  public function testIntegerFieldValueWithLessThanAndGreaterThan() {
    // Add an integer field to the node.
    $field_storage = FieldStorageConfig::create([
      'field_name' => 'field_integer',
      'entity_type' => 'node',
      'type' => 'integer',
      'cardinality' => -1,
    ]);
    $field_storage->save();
    $field = FieldConfig::create([
      'field_storage' => $field_storage,
      'field_name' => 'field_integer',
      'bundle' => 'page',
      'translatable' => FALSE,
    ]);
    $field->save();

    $policy = AccessPolicy::create([
      'id' => 'integer_between',
      'label' => 'Access policy with between values.',
      'access_rules' => [],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ]);
    $policy->save();

    // Add a less than operator.
    $policy->addAccessRule('field_value:node:field_integer', [
      'entity_field' => 'field_integer',
      'target' => 'value',
      'value' => [
        ['value' => 100],
      ],
      'operator' => '<',
      'query' => TRUE,
    ]);
    $policy->save();

    // Add a greater than operator.
    $policy->addAccessRule('field_value:node:field_integer', [
      'entity_field' => 'field_integer',
      'target' => 'value',
      'value' => [
        ['value' => 50],
      ],
      'operator' => '>',
      'query' => TRUE,
    ]);
    $policy->save();

    // Add a greater than operator.
    $policy->addAccessRule('field_value:node:field_integer', [
      'entity_field' => 'field_integer',
      'target' => 'value',
      'value' => [
        ['value' => 101],
      ],
      'operator' => '!=',
      'query' => TRUE,
    ]);
    $policy->save();

    // 75 is between 50 and 100 and should be visible.
    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1 (With int: 75)',
      'type' => 'page',
      'status' => 1,
      'field_integer' => [
        75,
      ],
    ]);

    // 25 is less than 50 and should not be visible.
    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2 (With int: 25)',
      'type' => 'page',
      'status' => 1,
      'field_integer' => [
        25,
      ],
    ]);

    // 125 is greater than 100 and should not be visible.
    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3 (With int: 125)',
      'type' => 'page',
      'status' => 1,
      'field_integer' => [
        125,
      ],
    ]);

    // 101 is not allowed and should not be visible.
    $node_4 = $this->drupalCreateNode([
      'title' => 'Node 4 (With int: 101)',
      'type' => 'page',
      'status' => 1,
      'field_integer' => [
        101,
      ],
    ]);

    $this->assignAccessPolicy($node_1, 'integer_between');
    $this->assignAccessPolicy($node_2, 'integer_between');
    $this->assignAccessPolicy($node_3, 'integer_between');
    $this->assignAccessPolicy($node_4, 'integer_between');

    // We're not comparing field values with a user so it's not necessary to set
    // it.
    $admin_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view integer_between content',
    ]);
    $admin_user->save();

    $this->drupalLogin($admin_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_4->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1 (With int: 75)');
    $this->assertSession()->pageTextNotContains('Node 2 (With int: 25)');
    $this->assertSession()->pageTextNotContains('Node 3 (With int: 125)');
    $this->assertSession()->pageTextNotContains('Node 4 (With int: 101)');
  }

  /**
   * Tests field value contains and not contains.
   */
  public function testFieldValueContainsAndNotContains() {
    $policy = AccessPolicy::create([
      'id' => 'string_contains',
      'label' => 'Access policy contains',
      'access_rules' => [],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ]);
    $policy->save();

    // Add a contains operator.
    $policy->addAccessRule('field_value:node:field_text', [
      'entity_field' => 'field_text',
      'target' => 'value',
      'value' => [
        ['value' => 'Foo'],
      ],
      'operator' => 'contains',
      'query' => TRUE,
    ]);
    $policy->save();

    // Add a not contains operator.
    $policy->addAccessRule('field_value:node:field_text', [
      'entity_field' => 'field_text',
      'target' => 'value',
      'value' => [
        ['value' => 'Bar'],
      ],
      'operator' => 'not contains',
      'query' => TRUE,
    ]);
    $policy->save();

    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1 (With string: Foo)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Foo',
      ],
    ]);

    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2 (With string: Bar)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Bar',
      ],
    ]);

    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3 (With string: Foo Bar)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Foo Bar',
      ],
    ]);

    // Because this has one value that matches it should be visible.
    $node_4 = $this->drupalCreateNode([
      'title' => 'Node 4 (With multiple values - allowed)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Bar',
        'Text with Foo',
      ],
    ]);

    // Because this does not have any values that match it should not be
    // visible.
    $node_5 = $this->drupalCreateNode([
      'title' => 'Node 5 (With multiple values - denied)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Bar',
        'Text with Bar again',
      ],
    ]);

    $this->assignAccessPolicy($node_1, 'string_contains');
    $this->assignAccessPolicy($node_2, 'string_contains');
    $this->assignAccessPolicy($node_3, 'string_contains');
    $this->assignAccessPolicy($node_4, 'string_contains');
    $this->assignAccessPolicy($node_5, 'string_contains');

    // We're not comparing field values with a user so it's not necessary to set
    // it.
    $admin_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view string_contains content',
    ]);
    $admin_user->save();

    $this->drupalLogin($admin_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_4->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_5->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1 (With string: Foo)');
    $this->assertSession()->pageTextNotContains('Node 2 (With string: Bar)');
    $this->assertSession()->pageTextNotContains('Node 3 (With string: Foo Bar)');
    $this->assertSession()->pageTextContainsOnce('Node 4 (With multiple values - allowed)');
    $this->assertSession()->pageTextNotContains('Node 5 (With multiple values - denied)');
  }

  /**
   * Tests field value starts with and not starts with.
   */
  public function testFieldValueStartsWithAndNotStartsWith() {
    $policy = AccessPolicy::create([
      'id' => 'string_starts_with',
      'label' => 'Access policy starts with',
      'access_rules' => [],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ]);
    $policy->save();

    $policy->addAccessRule('field_value:node:field_text', [
      'entity_field' => 'field_text',
      'target' => 'value',
      'value' => [
        ['value' => 'Foo'],
      ],
      'operator' => 'starts with',
      'query' => TRUE,
    ]);
    $policy->save();

    $policy->addAccessRule('field_value:node:field_text', [
      'entity_field' => 'field_text',
      'target' => 'value',
      'value' => [
        ['value' => 'Bar'],
      ],
      'operator' => 'not starts with',
      'query' => TRUE,
    ]);
    $policy->save();

    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1 (With start: Foo)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Foo with text',
      ],
    ]);

    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2 (With start: Bar)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Bar with text',
      ],
    ]);

    // Because this has one value that matches it should be visible.
    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3 (With multiple values - allowed)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Foo with text',
        'Bar with text',
      ],
    ]);

    // Because this does not have any values that match it should not be
    // visible.
    $node_4 = $this->drupalCreateNode([
      'title' => 'Node 4 (With multiple values - denied)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Foo',
        'Bar with text again',
      ],
    ]);

    $this->assignAccessPolicy($node_1, 'string_starts_with');
    $this->assignAccessPolicy($node_2, 'string_starts_with');
    $this->assignAccessPolicy($node_3, 'string_starts_with');
    $this->assignAccessPolicy($node_4, 'string_starts_with');

    // We're not comparing field values with a user so it's not necessary to set
    // it.
    $admin_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view string_starts_with content',
    ]);
    $admin_user->save();

    $this->drupalLogin($admin_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_4->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1 (With start: Foo)');
    $this->assertSession()->pageTextNotContains('Node 2 (With start: Bar)');
    $this->assertSession()->pageTextContainsOnce('Node 3 (With multiple values - allowed)');
    $this->assertSession()->pageTextNotContains('Node 4 (With multiple values - denied)');
  }

  /**
   * Tests field value ends with and not ends with.
   */
  public function testFieldValueEndsWithAndNotEndsWith() {
    $policy = AccessPolicy::create([
      'id' => 'string_ends_with',
      'label' => 'Access policy ends with',
      'access_rules' => [],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ]);
    $policy->save();

    $policy->addAccessRule('field_value:node:field_text', [
      'entity_field' => 'field_text',
      'target' => 'value',
      'value' => [
        ['value' => 'Foo'],
      ],
      'operator' => 'ends with',
      'query' => TRUE,
    ]);
    $policy->save();

    $policy->addAccessRule('field_value:node:field_text', [
      'entity_field' => 'field_text',
      'target' => 'value',
      'value' => [
        ['value' => 'Bar'],
      ],
      'operator' => 'not ends with',
      'query' => TRUE,
    ]);
    $policy->save();

    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1 (With end: Foo)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Foo',
      ],
    ]);

    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2 (With end: Bar)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Bar',
      ],
    ]);

    // Because this has one value that matches it should be visible.
    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3 (With multiple values - allowed)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Text with Foo',
        'Text with Bar',
      ],
    ]);

    // Because this does not have any values that match it should not be
    // visible.
    $node_4 = $this->drupalCreateNode([
      'title' => 'Node 4 (With multiple values - denied)',
      'type' => 'page',
      'status' => 1,
      'field_text' => [
        'Foo with text',
        'Bar with text again',
      ],
    ]);

    $this->assignAccessPolicy($node_1, 'string_ends_with');
    $this->assignAccessPolicy($node_2, 'string_ends_with');
    $this->assignAccessPolicy($node_3, 'string_ends_with');
    $this->assignAccessPolicy($node_4, 'string_ends_with');

    // We're not comparing field values with a user so it's not necessary to set
    // it.
    $admin_user = $this->drupalCreateUser([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view string_ends_with content',
    ]);
    $admin_user->save();

    $this->drupalLogin($admin_user);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_4->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1 (With end: Foo)');
    $this->assertSession()->pageTextNotContains('Node 2 (With end: Bar)');
    $this->assertSession()->pageTextContainsOnce('Node 3 (With multiple values - allowed)');
    $this->assertSession()->pageTextNotContains('Node 4 (With multiple values - denied)');
  }

}
