<?php

namespace Drupal\Tests\access_policy\Functional;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\Tests\field\Traits\EntityReferenceTestTrait;
use Drupal\access_policy\Entity\AccessPolicy;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * Tests the UserEntityReference access rule.
 *
 * @group access_policy
 */
class AccessRuleUserEntityReferenceTest extends AccessPolicyTestBase {

  use UserCreationTrait;
  use EntityReferenceTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'access_policy',
    'access_policy_test',
    'filter',
    'node',
    'field',
    'user',
    'datetime',
    'views',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Set to TRUE to strict check all configuration saved.
   *
   * @var bool
   *
   * @see \Drupal\Core\Config\Development\ConfigSchemaChecker
   */
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setup();

    AccessPolicy::create([
      'id' => 'user_entity_reference',
      'label' => 'User entity reference Access Policy',
      'access_rules' => [
        'user_entity_reference' => [
          'plugin_id' => 'user_entity_reference',
          'settings' => [
            'entity_field' => 'field_entity_reference',
            'query' => TRUE,
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ])->save();

    // Create the entity reference field on the user.
    $field_storage = FieldStorageConfig::create([
      'field_name' => 'field_entity_reference',
      'entity_type' => 'user',
      'type' => 'entity_reference',
      'cardinality' => -1,
      'settings' => [
        'target_type' => 'node',
      ],
    ]);
    $field_storage->save();

    $field = FieldConfig::create([
      'field_storage' => $field_storage,
      'field_name' => 'field_entity_reference',
      'bundle' => 'user',
      'translatable' => FALSE,
    ]);
    $field->save();

  }

  /**
   * Tests viewing a node that is referenced by the user.
   */
  public function testViewEntityWithUserEntityReferenceAccessRule() {

    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
    ]);

    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2',
      'type' => 'page',
      'status' => 1,
    ]);

    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3',
      'type' => 'page',
      'status' => 1,
    ]);

    $this->assignAccessPolicy($node_1, 'user_entity_reference');
    $this->assignAccessPolicy($node_2, 'user_entity_reference');
    $this->assignAccessPolicy($node_3, 'user_entity_reference');

    // Create a user who has a reference to some of the nodes above.
    $rid = $this->createRole([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view user_entity_reference content',
    ]);

    $web_user_1 = $this->createUser();
    $web_user_1->addRole($rid);
    $web_user_1->set('field_entity_reference', [
      $node_1,
      $node_2,
    ]);
    $web_user_1->save();

    $this->drupalLogin($web_user_1);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1');
    $this->assertSession()->pageTextContainsOnce('Node 2');
    $this->assertSession()->pageTextNotContains('Node 3');
  }

}
