<?php

namespace Drupal\Tests\access_policy\Functional;

use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\Tests\field\Traits\EntityReferenceTestTrait;
use Drupal\access_policy\Entity\AccessPolicy;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\user\Entity\Role;

/**
 * Ensures that listings only display entities that a user is allowed to view.
 *
 * @group access_policy
 */
class AccessRuleUserReferenceTest extends AccessPolicyTestBase {

  use UserCreationTrait;
  use EntityReferenceTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'access_policy',
    'access_policy_test',
    'filter',
    'node',
    'field',
    'user',
    'datetime',
    'views',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Set to TRUE to strict check all configuration saved.
   *
   * @var bool
   *
   * @see \Drupal\Core\Config\Development\ConfigSchemaChecker
   */
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setup();

    AccessPolicy::create([
      'id' => 'user_reference',
      'label' => 'User reference Access Policy',
      'access_rules' => [
        'user_reference' => [
          'plugin_id' => 'user_reference',
          'settings' => [
            'entity_field' => 'field_user_reference',
            'query' => TRUE,
          ],
        ],
      ],
      'target_entity_type_id' => 'node',
      'target_bundles' => ['page'],
    ])->save();

    // Create the user reference field.
    $field_storage = FieldStorageConfig::create([
      'field_name' => 'field_user_reference',
      'entity_type' => 'node',
      'type' => 'entity_reference',
      'cardinality' => -1,
      'settings' => [
        'target_type' => 'user',
      ],
    ]);
    $field_storage->save();

    $field = FieldConfig::create([
      'field_storage' => $field_storage,
      'field_name' => 'field_user_reference',
      'bundle' => 'page',
      'translatable' => FALSE,
    ]);
    $field->save();

  }

  /**
   * Tests viewing a node that contains matching entity references.
   */
  public function testViewEntityWithUserReferenceAccessRule() {
    $rid = $this->createRole([
      'access content',
      'access administration pages',
      'access content overview',
      'administer nodes',
      'view user_reference content',
    ]);

    // Create some users to test against.
    $web_user_1 = $this->createUser();
    $web_user_1->addRole($rid);
    $web_user_1->save();

    $web_user_2 = $this->createUser();
    $web_user_2->addRole($rid);
    $web_user_2->save();

    $node_1 = $this->drupalCreateNode([
      'title' => 'Node 1',
      'type' => 'page',
      'status' => 1,
      'field_user_reference' => [
        $web_user_1
      ],
    ]);

    $node_2 = $this->drupalCreateNode([
      'title' => 'Node 2',
      'type' => 'page',
      'status' => 1,
      'field_user_reference' => [
        $web_user_1,
        $web_user_2
      ],
    ]);

    $node_3 = $this->drupalCreateNode([
      'title' => 'Node 3',
      'type' => 'page',
      'status' => 1,
    ]);

    $this->assignAccessPolicy($node_1, 'user_reference');
    $this->assignAccessPolicy($node_2, 'user_reference');
    $this->assignAccessPolicy($node_3, 'user_reference');

    // Login as a user with a matching entity reference. They should be granted
    // access.
    $this->drupalLogin($web_user_1);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContainsOnce('Node 1');
    $this->assertSession()->pageTextContainsOnce('Node 2');
    $this->assertSession()->pageTextNotContains('Node 3');

    $this->drupalLogin($web_user_2);
    $this->drupalGet('node/' . $node_1->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('node/' . $node_2->id());
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('node/' . $node_3->id());
    $this->assertSession()->statusCodeEquals(403);

    $this->drupalGet('admin/content');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextNotContains('Node 1');
    $this->assertSession()->pageTextContainsOnce('Node 2');
    $this->assertSession()->pageTextNotContains('Node 3');
  }

}
